unit SearchEngineFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Menus, SearchEditFrm, ExtCtrls;

type
  TSearchEngineForm = class(TForm)
    lstvwSearchEngines: TListView;
    PopupMenu1: TPopupMenu;
    btnInsert: TButton;
    btnDelete: TButton;
    btnOk: TButton;
    btnCancel: TButton;
    chkbxUseSameWindow: TCheckBox;
    Insert1: TMenuItem;
    Delete1: TMenuItem;
    btnAbout: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnInsertClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnDeleteClick(Sender: TObject);
    procedure lstvwSearchEnginesSelectItem(Sender: TObject;
      Item: TListItem; Selected: Boolean);
    procedure lstvwSearchEnginesDblClick(Sender: TObject);
    procedure lstvwSearchEnginesDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure lstvwSearchEnginesDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure btnAboutClick(Sender: TObject);
  private
    FSearchEditForm: TSearchEditForm;
  end;

implementation

uses
  IniFiles, SearchWizFrm, SearchAboutFrm;

{$R *.DFM}

{ TSearchEngineForm }

procedure TSearchEngineForm.FormCreate(Sender: TObject);
var
  EngineList: TStrings;
  I: Integer;
begin
  // Load in the search engines
  EngineList := TStringList.Create;
  try
    LoadSearchEngines(EngineList);
    lstvwSearchEngines.Items.BeginUpdate;
    try
      for I := 0 to EngineList.Count - 1 do
      begin
        with lstvwSearchEngines.Items.Add do
        begin
          Caption := EngineList.Names[I];
          SubItems.Add(EngineList.Values[EngineList.Names[I]]);
        end;
      end;
      // Set the first one as selected
      if lstvwSearchEngines.Items.Count > 0 then
        lstvwSearchEngines.Selected := lstvwSearchEngines.Items[0];
    finally
      lstvwSearchEngines.Items.EndUpdate;
    end;
  finally
    EngineList.Free;
  end;
  FSearchEditForm := TSearchEditForm.Create(Self);
end;

procedure TSearchEngineForm.FormDestroy(Sender: TObject);
begin
  FSearchEditForm.Free;
end;

procedure TSearchEngineForm.btnInsertClick(Sender: TObject);
var
  NewItem: TListItem;
begin
  FSearchEditForm.memUrl.Text := '';
  FSearchEditForm.edtEngineName.Text := '';
  if FSearchEditForm.ShowModal = mrOk then
  begin
    NewItem := lstvwSearchEngines.Items.Add;
    with NewItem do
    begin
      Caption := FSearchEditForm.edtEngineName.Text;
      SubItems.Add(FSearchEditForm.memUrl.Text);
    end;
    lstvwSearchEngines.Selected := NewItem;
    btnDelete.Enabled := True;
  end;
end;

procedure TSearchEngineForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    Close;
end;

procedure TSearchEngineForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
  SearchEngineList: TStringList;
  I: Integer;
begin
  if ModalResult = mrOk then
  begin
    // Save the search engines
    SearchEngineList := TStringList.Create;
    try
      for I := 0 to lstvwSearchEngines.Items.Count - 1 do
      begin
        SearchEngineList.Values[lstvwSearchEngines.Items[I].Caption] :=
          lstvwSearchEngines.Items[I].SubItems[0];
      end;
      SaveSearchEngines(SearchEngineList);
    finally
      SearchEngineList.Free;
    end;
  end;
end;

procedure TSearchEngineForm.btnDeleteClick(Sender: TObject);
begin
  if lstvwSearchEngines.Selected <> nil then
    lstvwSearchEngines.Selected.Delete;
  if lstvwSearchEngines.Items.Count > 0 then
    lstvwSearchEngines.Selected := lstvwSearchEngines.Items[0]
  else
    btnDelete.Enabled := False;
end;

procedure TSearchEngineForm.lstvwSearchEnginesSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  if (Item = nil) or (not Selected) then
    Exit;
  btnDelete.Enabled := True;
end;

procedure TSearchEngineForm.lstvwSearchEnginesDblClick(Sender: TObject);
begin
  if lstvwSearchEngines.Selected <> nil then
  begin
    with FSearchEditForm do
    begin
      edtEngineName.Text := lstvwSearchEngines.Selected.Caption;
      memUrl.Text := lstvwSearchEngines.Selected.SubItems[0];
      if ShowModal = mrOk then
      begin
        lstvwSearchEngines.Selected.Caption := edtEngineName.Text;
        lstvwSearchEngines.Selected.SubItems[0] := memUrl.Text;
      end;
    end;
  end;
end;

procedure TSearchEngineForm.lstvwSearchEnginesDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
var
  Item: TListItem;
begin
  Accept := (Sender = lstvwSearchEngines) and (Source = lstvwSearchEngines);
  if Accept then
  begin
    // Can't drag/drop on the same spot
    Item := lstvwSearchEngines.GetItemAt(X, Y);
    if (Item = lstvwSearchEngines.Selected) or (Item = nil) then
      Accept := False;
  end;
end;

procedure TSearchEngineForm.lstvwSearchEnginesDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  Item: TListItem;
  Caption, SubItem: string;
  I: Integer;
begin
  // Insert the item - do a insert above instead
  // of a switch.
  if (Sender = lstvwSearchEngines) and (Source = lstvwSearchEngines) then
  begin
    lstvwSearchEngines.Items.BeginUpdate;
    try
      Item := lstvwSearchEngines.GetItemAt(X, Y);
      if (lstvwSearchEngines.Selected <> nil) and
        (Item <> lstvwSearchEngines.Selected) and (Item <> nil) then
      begin
        // We are either moving the selected on up or down.
        if lstvwSearchEngines.Selected.Index < Item.Index then
        begin
          // Moving down - switch each of the captions until
          // we are 2 up from the selected index - 2 because we
          // want to be one above the selected (as designed)
          for I := lstvwSearchEngines.Selected.Index to Item.Index - 2 do
          begin
            Caption := lstvwSearchEngines.Items[I].Caption;
            SubItem := lstvwSearchEngines.Items[I].SubItems[0];
            lstvwSearchEngines.Items[I].Caption :=
              lstvwSearchEngines.Items[I+1].Caption;
            lstvwSearchEngines.Items[I].SubItems[0] :=
              lstvwSearchEngines.Items[I+1].SubItems[0];
            lstvwSearchEngines.Items[I+1].Caption := Caption;
            lstvwSearchEngines.Items[I+1].SubItems[0] :=SubItem;
          end;
        end
        else
        begin
          // Moving up - exactly the opposite of above
          for I := lstvwSearchEngines.Selected.Index downto Item.Index + 1 do
          begin
            Caption := lstvwSearchEngines.Items[I].Caption;
            SubItem := lstvwSearchEngines.Items[I].SubItems[0];
            lstvwSearchEngines.Items[I].Caption :=
              lstvwSearchEngines.Items[I-1].Caption;
            lstvwSearchEngines.Items[I].SubItems[0] :=
              lstvwSearchEngines.Items[I-1].SubItems[0];
            lstvwSearchEngines.Items[I-1].Caption := Caption;
            lstvwSearchEngines.Items[I-1].SubItems[0] :=SubItem;
          end;
        end;
      end;
    finally
      lstvwSearchEngines.Items.EndUpdate;
    end;
  end;
end;

procedure TSearchEngineForm.btnAboutClick(Sender: TObject);
begin
  with TAboutForm.Create(Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

end.
