unit SearchWizFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DockForm, Buttons, IniFiles, ExtCtrls, OleCtrls,
  DesignIntf, ActnList, Menus, ImgList, SHDocVw, StdActns;

type
  TDockState = (dsDocked, dsUndocked);

  TSearchWizForm = class(TDockableForm)
    pnlTop: TPanel;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    btnSearch: TSpeedButton;               
    btnCustomize: TSpeedButton;
    btnUpDown: TSpeedButton;
    cmbxEngines: TComboBox;
    cmbxQuery: TComboBox;
    popRightClick: TPopupMenu;
    mnuDockable: TMenuItem;
    mnuStayOnTop: TMenuItem;
    btnRefresh: TSpeedButton;
    btnBack: TSpeedButton;
    btnForward: TSpeedButton;
    imglstUpDown: TImageList;
    EditCopy1: TEditCopy;
    EditCut1: TEditCut;
    EditPaste1: TEditPaste;
    EditSelectAll1: TEditSelectAll;
    EditUndo1: TEditUndo;
    EditSelectAll2: TEditSelectAll;
    actnBack: TAction;
    actnForward: TAction;
    actnRefresh: TAction;
    N1: TMenuItem;
    About1: TMenuItem;
    procedure btnSearchClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCustomizeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnUpDownClick(Sender: TObject);
    procedure FormStartDock(Sender: TObject;
      var DragObject: TDragDockObject);
    procedure FormEndDock(Sender, Target: TObject; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EditCopy1Execute(Sender: TObject);
    procedure EditCut1Execute(Sender: TObject);
    procedure EditPaste1Execute(Sender: TObject);
    procedure EditUndo1Execute(Sender: TObject);
    procedure EditSelectAll2Execute(Sender: TObject);
    procedure actnBackExecute(Sender: TObject);
    procedure actnForwardExecute(Sender: TObject);
    procedure actnRefreshExecute(Sender: TObject);
    procedure actnRefreshUpdate(Sender: TObject);
    procedure actnBackUpdate(Sender: TObject);
    procedure actnForwardUpdate(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    FWebBrowserShowing: Boolean;
    FSearchEngineList: TStrings;
    FLastBrowserHeight: Integer;
    FWebBrowser: TWebBrowser;
    FUseSameWindow: Boolean;

    FHistoryList: TStringList;
    FHistoryIndex: Integer;

    // fix for "losing" the last URL when docked/undocked
    FLastURL: string;
    FDockState: TDockState;

    procedure SetSearchComboBox;
    procedure SetWebBrowserShowing(const Value: Boolean);
    procedure CMDialogKey(var Message: TCMDialogKey);  message CM_DIALOGKEY;
    procedure SetUseSameWindow(const Value: Boolean);
    procedure WebBrowserBeforeNavigate2(Sender: TObject;
      const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
      Headers: OleVariant; var Cancel: WordBool);
    procedure BrowseToHistoryIndex;
  protected
    property UseSameWindow: Boolean read FUseSameWindow write SetUseSameWindow;
    property WebBrowserShowing: Boolean read FWebBrowserShowing write SetWebBrowserShowing;
  public
    procedure LoadWindowState(Desktop: TMemIniFile); override;
    procedure SaveWindowState(Desktop: TMemIniFile; isProject: Boolean); override;
  end;

var
  SearchWizForm: TSearchWizForm = nil;

procedure LoadSearchEngines(var EngineList: TStrings);
procedure SaveSearchEngines(const EngineList: TStringList);
procedure ShowAboutWebSearch;

resourcestring
  // Sections in the ini file
  CIniSection = 'SearchWizard';
  CIniSearchEngines = 'SearchWizard SearchEngines';

implementation

uses
  ShellApi, SearchEngineFrm, DeskUtil;

resourcestring
  CMSDNName = 'MSDN';
  CGoogleName = 'Google';
  CAltaVistaName = 'AltaVista';
  CBorlandCommunityName = 'Borland Community';
  CDejaName = 'Deja.com';
  CYahooName = 'Yahoo!';
  CExciteName = 'Excite';
  CNorthernLightName = 'Northern Light';

  CIniLastSearchQuery = 'Last Query';
  CIniLastSearchEngine = 'Last Search Engine';
  CIniSavedQueries = 'SearchWizard Saved Queries';
  CIniUseSameWindow = 'Use Same Window';
  CIniWebBrowserShowing = 'WebBrowser Showing';
  CAboutWebSearch = 'About Stuff Here';

const 
  CMSDNValue =
    'http://search.microsoft.com/us/dev/default.asp?so=RECCNT&qu=' +
    '%s' + // Search string goes here!
    '&boolean=ALL&chkM=on&chkM_0=on&chkM_1=on&chkM_2=on&chkM_3=on&chkM_4='
    + 'on&chkM_5=on&chkM_6=on&chkM_7=on&chkM_8=on&chkM_9=on&chkM_10=on&chkM_11='
    +'on&chkM_12=on&chkM_13=on&chkM_14=on&chkM_15=on&chkM_16=on&chkM_17=on&chkM_18='
    +'on&chkS=on&chkS_0=on&chkS_1=on&chkS_2=on&chkS_3=on&chkS_4=on&chkS_5=on&chkS_6='
    +'on&chkS_7=on&chkS_8=on&chkS_9=on&chkS_10=on&chkS_11=on&chkS_12=on&chkS_13=on&chkS_14='
    +'on&chkS_15=on&chkA_0=on&chkA_1=on&chkA_2=on&chkA_3=on&chkA_4=on&chkA_5=on&chkA_6=on&chkP_0='
    +'on&chkP_1=on&chkP_2=on&chkP_3=on&chkP_4=on&chkP_5=on&chkP_6=on&chkP_7=on&chkP_8=on&chkP_9=on&chkP_10='
    +'on&chkP_11=on&chkP_12=on&chkP_13=on&chkP_14=on&chkP_15=on&chkP_16=on&chkK=on&p=1&nq=NEW';
  CGoogleValue = 'http://www.google.com/search?q=%s';
  CAltaVistaValue = 'http://www.altavista.com/cgi-bin/query?q=%s';
  CAltaVistaSmallValue = 'http://www.altavista.com/cgi-bin/query?pg=n200&stype=stext&user=msn&poa=msn&q=%s&stq=0';

  CBorlandCommunityValue = 'http://search.borland.com/query.html?col=nowebdsp&qt=%s&qs=&qc=&pw='+
    '100%25&ws=0&la=&qm=0&st=1&nh=10&lk=1&rf=0&oq=&rq=0';

  CDejaValue = 'http://www.deja.com/dnquery.xp?preserve=1&QRY=%s&ST=MS&svcclass=dnserver&DBS=2';

  CYahooValue = 'http://search.yahoo.com/bin/search?p=%s';
  CYahooSmallValue = 'http://search.yahoo.com/search?o=1&m=i&a=fd&p=%s';

  CExciteValue = 'http://search.excite.com/search.gw?search=%s';
  CExciteSmallValue = 'http://search.excite.com/search.gw?trace=1&look=ie40&search=%s';

  CNorthernLightValue = 'http://www.northernlight.com/nlquery.fcg?cb=0&qr=%s&orl=';
  CNorthernLightSmallValue = 'http://www.northernlight.com/nlquery.fcg?qr=%s&si=&cb=0&cc=&us=025&dx=1007#resultsresults'; 

  CDefaultBrowserHeight = 300;
  CDefaultTopPanelHeight = 70;
{$R *.DFM}

{ From HTTPApp.pas }
function HTTPEncode(const AStr: String): string;
const
  NoConversion = ['A'..'Z','a'..'z','*','@','.','_','-',
                  '0'..'9','$','!','''','(',')'];
var
  Sp, Rp: PChar;
begin
  SetLength(Result, Length(AStr) * 3);
  Sp := PChar(AStr);
  Rp := PChar(Result);
  while Sp^ <> #0 do
  begin
    if Sp^ in NoConversion then
      Rp^ := Sp^
    else
      if Sp^ = ' ' then
        Rp^ := '+'
      else
      begin
        FormatBuf(Rp^, 3, '%%%.2x', 6, [Ord(Sp^)]);
        Inc(Rp,2);
      end;
    Inc(Rp);
    Inc(Sp);
  end;
  SetLength(Result, Rp - PChar(Result));
end;

procedure ShowAboutWebSearch;
begin
  ShowMessage(CAboutWebSearch);
end;

procedure LoadSearchEngines(var EngineList: TStrings);
var
  TempList: TStringList;
  I: Integer;
begin
  if Assigned(IDEIniFile) then
  begin
    EngineList.Clear;
    with IDEIniFile do
    begin
      if SectionExists(CIniSearchEngines) then
      begin
        TempList := TStringList.Create;
        try
          ReadSection(CIniSearchEngines, TempList);
          for I := 0 to TempList.Count - 1 do
          begin
            // Save the search engines in a Name=Value EngineList
            // string list. Such as:
            // Google=http://www.google.com/search?q=corbin
            EngineList.Values[TempList[I]] := ReadString(CiniSearchEngines,
              TempList[I], '');
          end;
        finally
          TempList.Free;
        end;
      end;
    end;
    // Add some defaults if there were none found
    if EngineList.Count <= 0 then
    begin
      EngineList.Values[CMSDNName] := CMSDNValue;
      EngineList.Values[CGoogleName] := CGoogleValue;
      EngineList.Values[CBorlandCommunityName] := CBorlandCommunityValue;
      EngineList.Values[CDejaName] := CDejaValue;

      // "Small" values are like the IE "search" button
      if (SearchWizForm <> nil) and (SearchWizForm.UseSameWindow) then
      begin
        EngineList.Values[CAltaVistaName] := CAltaVistaSmallValue;
        EngineList.Values[CYahooName] := CYahooSmallValue;
        EngineList.Values[CExciteName] := CExciteSmallValue;
        EngineList.Values[CNorthernLightName] := CNorthernLightSmallValue;
      end else
      begin
        EngineList.Values[CAltaVistaName] := CAltaVistaValue;
        EngineList.Values[CYahooName] := CYahooValue;
        EngineList.Values[CExciteName] := CExciteValue;
        EngineList.Values[CNorthernLightName] := CNorthernLightValue;
      end;
    end;
  end;
end;

procedure SaveSearchEngines(const EngineList: TStringList);
var
  I: Integer;
begin
  if Assigned(IDEIniFile) then
  begin
    with IDEIniFile do
    begin
      EraseSection(CIniSearchEngines);
      for I := 0 to EngineList.Count - 1 do
        WriteString(CIniSearchEngines, EngineList.Names[I], EngineList.Values[EngineList.Names[I]]);
    end;
  end;
end;

procedure TSearchWizForm.btnSearchClick(Sender: TObject);
var
  URL: string;
  Loc: Integer;
begin
  if cmbxQuery.Text <> '' then
  begin
    // Limit to a history of 50 strings.
    if cmbxQuery.Items.Count >= 50 then
      cmbxQuery.Items.Delete(50);
    if cmbxQuery.Items.IndexOf(cmbxQuery.Text) = -1 then
      cmbxQuery.Items.Insert(0, cmbxQuery.Text);

    with cmbxEngines do
      URL := FSearchEngineList.Values[Items[ItemIndex]];
    Loc := Pos('%s', URL);
    if Loc <> 0 then
    begin
      URL := Copy(URL, 1, Loc - 1) + HTTPEncode(cmbxQuery.Text) +
        Copy(URL, Loc + 2, Length(URL) - (Loc + 1));
    end;
    if FUseSameWindow and Assigned(FWebBrowser) then
    begin
      // Make sure the browser is showing; otherwise, we will get errors
      // from the internal browser (it needs a window to be visible to
      // actually browse)
      if not FWebBrowserShowing then
        WebBrowserShowing := True;
      FWebBrowser.Navigate(URL);
    end
    else
      ShellExecute(Handle, 'open', PChar(URL), nil, nil, SW_SHOW);
    cmbxQuery.SelectAll;
    if Active then
      SetFocusedControl(cmbxQuery);
  end;
end;

procedure TSearchWizForm.FormDestroy(Sender: TObject);
begin
  FSearchEngineList.Free;
  FHistoryList.Free;
  if Assigned(FWebBrowser) then
    FWebBrowser.Free;
  inherited;
  SearchWizForm := nil;
end;

procedure TSearchWizForm.SetSearchComboBox;
var
  I: Integer;
  LastEngine: string;
  LastEngineIndex: Integer;
begin
  // Save the last engine that the user used
  if cmbxEngines.ItemIndex > 0 then
    LastEngine := cmbxEngines.Items[cmbxEngines.ItemIndex];

  cmbxEngines.Items.Clear;
  LoadSearchEngines(FSearchEngineList);
  for I := 0 to FSearchEngineList.Count - 1 do
    cmbxEngines.Items.Add(FSearchEngineList.Names[I]);
  LastEngineIndex := cmbxEngines.Items.IndexOf(LastEngine);
  if LastEngineIndex <> -1 then
    cmbxEngines.ItemIndex := LastEngineIndex;
  if (cmbxEngines.ItemIndex < 0) and (cmbxEngines.Items.Count > 0) then
    cmbxEngines.ItemIndex := 0;
end;

procedure TSearchWizForm.btnCustomizeClick(Sender: TObject);
begin
  with TSearchEngineForm.Create(Self) do
  begin
    chkbxUseSameWindow.Checked := FUseSameWindow;
    chkbxUseSameWindow.Enabled := FWebBrowser <> nil;
    if ShowModal = mrOk then
    begin
      if Assigned(FWebBrowser) then
      begin
        // By setting the properties, subsequent
        // states will be set up via their 'get' procedures
        UseSameWindow := chkbxUseSameWindow.Checked;
        WebBrowserShowing := FUseSameWindow;
      end;
      SetSearchComboBox;
    end;
  end;
end;

procedure TSearchWizForm.LoadWindowState(Desktop: TMemIniFile);
var
  TempList: TStringList;
  I: Integer;
begin
  inherited LoadWindowState(Desktop);

  if Assigned(Desktop) then
  begin
    cmbxQuery.Text := Desktop.ReadString(CIniSection, CIniLastSearchQuery, '');
    if FSearchEngineList.IndexOfName(Desktop.ReadString(CIniSection, CIniLastSearchEngine, '')) >= 0 then
      cmbxEngines.ItemIndex := FSearchEngineList.IndexOfName(Desktop.ReadString(CIniSection, CIniLastSearchEngine, ''));
    cmbxQuery.Items.Clear;
    TempList := TStringList.Create;
    try
      Desktop.ReadSection(CIniSavedQueries, TempList);
      for I := 0 to TempList.Count - 1 do
      begin
        if Desktop.ReadString(CIniSavedQueries, TempList[I], '') <> '' then
          cmbxQuery.Items.Add(Desktop.ReadString(CIniSavedQueries, TempList[I], ''));
      end;
    finally
      TempList.Free;
    end;
    // Be sure to use properties, not fields, below
    UseSameWindow := Desktop.ReadBool(CIniSection, CIniUseSameWindow, True);
    WebBrowserShowing := Desktop.ReadBool(CIniSection, CIniWebBrowserShowing, True) and UseSameWindow;
  end;
end;

procedure TSearchWizForm.SaveWindowState(Desktop: TMemIniFile;
  isProject: Boolean);
var
  I: Integer;
begin
  inherited SaveWindowState(Desktop, IsProject);
  if SaveStateNecessary and Assigned(Desktop) then
  begin
    Desktop.WriteString(CIniSection, CIniLastSearchQuery, cmbxQuery.Text);
    with cmbxEngines do
      Desktop.WriteString(CIniSection, CIniLastSearchEngine, Items[ItemIndex]);
    Desktop.EraseSection(CIniSavedQueries);
    if IsProject then
      for I := 0 to cmbxQuery.Items.Count - 1 do
        Desktop.WriteString(CIniSavedQueries, IntToStr(I), cmbxQuery.Items[I]);
    Desktop.WriteBool(CIniSection, CIniUseSameWindow, FUseSameWindow);
    Desktop.WriteBool(CIniSection, CIniWebBrowserShowing, FWebBrowserShowing);
  end;
end;

procedure TSearchWizForm.FormCreate(Sender: TObject);
begin
  inherited;
  FLastURL := '';
  FDockState := dsUndocked;
  FLastBrowserHeight := CDefaultBrowserHeight;

  FSearchEngineList := TStringList.Create;
  FHistoryList := TStringList.Create;
  FHistoryIndex := -1;
  btnUpDown.Glyph := TBitmap.Create;

  try
    // TWebBrowser.Create will raise exception if IE is not on the machine
    FWebBrowser := TWebBrowser.Create(Self);
    FWebBrowser.Align := alClient;
    TOleControl(FWebBrowser).Parent := Self;
    FWebBrowser.OnBeforeNavigate2 := WebBrowserBeforeNavigate2;
    // Using the properties below will resize the form
    // to accomidate the WebBrowser
    WebBrowserShowing := True;
    UseSameWindow := True;
  except
    FWebBrowser := nil;
    // Hide the browser area and button's corresponding to it
    WebBrowserShowing := False;
    UseSameWindow := False;
  end;

  DeskSection := CIniSection;
  AutoSave := True;
end;

procedure TSearchWizForm.btnUpDownClick(Sender: TObject);
begin
  WebBrowserShowing := not WebBrowserShowing;
  // Below is a fix for the browser loosing the URL it was on
  // after being docked/undocked
  if FWebBrowserShowing and (FLastURL <> '') then
  begin
    try
      FWebBrowser.Navigate(FLastURL);
    except
    end;
    FLastURL := '';
  end;
end;

procedure TSearchWizForm.SetWebBrowserShowing(const Value: Boolean);
begin
  if not Assigned(FWebBrowser) then
  begin
    btnRefresh.Visible := False;
    btnBack.Visible := False;
    btnForward.Visible := False;
    btnUpDown.Visible := False
  end
  else
  begin
    TOleControl(FWebBrowser).Visible := Value;
    btnUpDown.Glyph.Canvas.FillRect(Rect(0, 0,
      btnUpDown.Glyph.Width, btnUpDown.Glyph.Height));
  end;

  // Showing the browser, if it is available
  if Value and Assigned(FWebBrowser) then
  begin
    // If we are docked, then we just show the WebBrowser
    // and the alClient align takes care of the rest (done above).
    // Otherwise, resize the form, since we are a floating window. 
    if not Assigned(HostDockSite) then
    begin
      if FLastBrowserHeight <= 0 then
        FLastBrowserHeight := CDefaultBrowserHeight;
      ClientHeight := pnlTop.Height + FLastBrowserHeight;
    end;
    imglstUpDown.GetBitmap(0, btnUpDown.Glyph);
  end
  else
  begin
    if Assigned(FWebBrowser) then
      imglstUpDown.GetBitmap(1, btnUpDown.Glyph);

    // Resize the form so we don't see the browser, if we aren't docked
    if not Assigned(HostDockSite) then
      ClientHeight := pnlTop.Height;
  end;
  FWebBrowserShowing := Value;
end;

procedure TSearchWizForm.FormStartDock(Sender: TObject;
  var DragObject: TDragDockObject);
begin
  inherited;
  if Assigned(FWebBrowser) then
    FLastURL := FWebBrowser.LocationURL;
end;

procedure TSearchWizForm.FormEndDock(Sender, Target: TObject; X,
  Y: Integer);
begin
  inherited;
  
  if (FDockState = dsUndocked) and Assigned(Target) then
    // We are docking now - make sure that our sizes are correct
    FormResize(Sender)
  else if (FDockState = dsDocked) and not Assigned(Target) then
    // Force the webbrowser to be correctly shown
    WebBrowserShowing := WebBrowserShowing
  else // Undocking/docking didn't happen - don't rebrowse
    FLastURL := '';

  if Assigned(FWebBrowser) and (FLastURL <> '') and FWebBrowserShowing then 
  begin
    FWebBrowser.Navigate(FLastURL);
    FLastURL := '';
  end;

  if not Assigned(Target) then
    FDockState := dsUndocked
  else
    FDockState := dsDocked;
end;

procedure TSearchWizForm.CMDialogKey(var Message: TCMDialogKey);
begin
  if Message.CharCode = VK_RETURN then
  begin
    Message.Result := 1;
    btnSearchClick(nil);
  end
  else
    inherited;
end;

procedure TSearchWizForm.FormResize(Sender: TObject);
begin
  inherited;

  if FUseSameWindow and Assigned(FWebBrowser) then
  begin
    // If we are docked, then we use just hide/show the
    // browser and let the size of the docked host handle how
    // big it is. In that case, don't save the "FLastBrowserHeight"
    if FWebBrowserShowing and not Assigned(HostDockSite) then
      FLastBrowserHeight := FWebBrowser.Height
  end;
end;

procedure TSearchWizForm.FormShow(Sender: TObject);
begin
  inherited;
  // Call this here so that the SearchWizForm variable is known to be valid.
  SetSearchComboBox;
end;

procedure TSearchWizForm.SetUseSameWindow(const Value: Boolean);
begin
  FUseSameWindow := Value;
  btnUpDown.Visible := Value;

  // If we are using a separte window for browsing,
  // or, the browser is nil, then hide the up/down button
  // and resize the panel to hide where that button was.
  if not Assigned(FWebBrowser) or (not FUseSameWindow) then
    pnlTop.Height := CDefaultTopPanelHeight  - btnForward.Height
  else
    pnlTop.Height := CDefaultTopPanelHeight;
end;

procedure TSearchWizForm.EditCopy1Execute(Sender: TObject);
begin
  SendMessage(cmbxQuery.Handle, WM_COPY, 0, 0);
end;

procedure TSearchWizForm.EditCut1Execute(Sender: TObject);
begin
  SendMessage(cmbxQuery.Handle, WM_CUT, 0, 0);
end;

procedure TSearchWizForm.EditPaste1Execute(Sender: TObject);
begin
  SendMessage(cmbxQuery.Handle, WM_PASTE, 0, 0);
end;

procedure TSearchWizForm.EditUndo1Execute(Sender: TObject);
begin
  SendMessage(cmbxQuery.Handle, WM_UNDO, 0, 0);
end;

procedure TSearchWizForm.EditSelectAll2Execute(Sender: TObject);
begin
  if cmbxQuery.Focused then
    cmbxQuery.SelectAll;
end;

procedure TSearchWizForm.WebBrowserBeforeNavigate2(Sender: TObject;
  const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
var
  NewIndex: Integer;
begin
  NewIndex := FHistoryList.IndexOf(URL);
  if NewIndex = -1 then
  begin
    // Delete history from current index + 1 all the way to the end
    Inc(FHistoryIndex); 
    if (FHistoryIndex >= 0) and (FHistoryIndex < FHistoryList.Count - 1) then
      while FHistoryList.Count > FHistoryIndex do
        FHistoryList.Delete(FHistoryIndex);
    FHistoryIndex := FHistoryList.Add(URL);
  end
  else
    FHistoryIndex := NewIndex;
end;

procedure TSearchWizForm.actnBackExecute(Sender: TObject);
begin
  Dec(FHistoryIndex);
  BrowseToHistoryIndex;
end;

procedure TSearchWizForm.actnForwardExecute(Sender: TObject);
begin
  Inc(FHistoryIndex);
  BrowseToHistoryIndex;
end;

procedure TSearchWizForm.actnRefreshExecute(Sender: TObject);
begin
  if Assigned(FWebBrowser) then
    try
      FWebBrowser.Refresh;
    except
    end;
end;

procedure TSearchWizForm.BrowseToHistoryIndex;
begin
  if Assigned(FWebBrowser) and (FWebBrowser.Visible) then
  begin
    if (FHistoryIndex >= 0) and (FHistoryIndex < FHistoryList.Count) then
      FWebBrowser.Navigate(FHistoryList[FHistoryIndex]);
  end;
end;

procedure TSearchWizForm.actnRefreshUpdate(Sender: TObject);
begin
  actnRefresh.Enabled := FWebBrowserShowing and Assigned(FWebBrowser)
    and (FWebBrowser.LocationURL <> '');
end;

procedure TSearchWizForm.actnBackUpdate(Sender: TObject);
begin
  actnBack.Enabled := FWebBrowserShowing and Assigned(FWebBrowser)
    and (FHistoryList.Count > 0) and (FHistoryIndex > 0);
end;

procedure TSearchWizForm.actnForwardUpdate(Sender: TObject);
begin
  actnForward.Enabled := FWebBrowserShowing and Assigned(FWebBrowser)
    and (FHistoryList.Count > 0) and (FHistoryIndex < FHistoryList.Count - 1);
end;

procedure TSearchWizForm.About1Click(Sender: TObject);
begin
  ShowAboutWebSearch;
end;

end.
